@@ -1,3 +1,17 @@
+# Copyright 2001-2004 The Apache Software Foundation
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
 package Apache::TestServer;
 
 use strict;
@@ -8,6 +22,7 @@ use Socket ();
 use File::Spec::Functions qw(catfile);
 
 use Apache::TestTrace;
+use Apache::TestRun;
 use Apache::TestConfig ();
 use Apache::TestRequest ();
 
@@ -38,16 +53,51 @@ sub new {
 
     $self->{port_counter} = $self->{config}->{vars}->{port};
 
+    $self;
+}
+
+# call this when you already know where httpd is
+sub post_config {
+    my($self) = @_;
+
     $self->{version} = $self->{config}->httpd_version || '';
     $self->{mpm}     = $self->{config}->httpd_mpm     || '';
-    ($self->{rev})   = $self->{version} =~ m:^Apache/(\d)\.:;
-    $self->{rev}   ||= 2;
+
+    # try to get the revision number from the standard Apache version
+    # string and various variations made by distributions which mangle
+    # that string
+
+    # Apache/2.0.50-dev
+    ($self->{rev})   = $self->{version} =~ m|^Apache/(\d)\.|;
+
+    # Apache-AdvancedExtranetServer/1.3.29 (Mandrake Linux/1mdk)
+    ($self->{rev}) ||= $self->{version} =~ m|^Apache.*?/(\d)\.|;
+
+    # IBM_HTTP_SERVER/1.3.19  Apache/1.3.20 (Unix)
+    ($self->{rev}) ||= $self->{version} =~ m|^.*?Apache.*?/(\d)\.|;
+
+    if ($self->{rev}) {
+        debug "Matched Apache revision $self->{version} $self->{rev}";
+    }
+    else {
+        # guessing is not good as it'll only mislead users
+        # and we can't die since a config object is required
+        # during Makefile.PL's write_perlscript when path to httpd may
+        # be unknown yet. so default to non-existing version 0 for now.
+        # and let TestRun.pm figure out the required pieces
+        debug "can't figure out Apache revision, from string: " .
+            "'$self->{version}', using a non-existing revision 0";
+        $self->{rev} = 0; # unknown
+    }
 
     $self;
 }
 
 sub version_of {
     my($self, $thing) = @_;
+    die "Can't figure out what Apache server generation we are running"
+        unless $self->{rev};
+
     $thing->{$self->{rev}};
 }
 
@@ -77,15 +127,21 @@ sub pid_file {
 
 sub dversion {
     my $self = shift;
-    "-DAPACHE$self->{rev}";
+    "-D APACHE$self->{rev}";
 }
 
 sub config_defines {
+    my $self = shift;
+
     my @defines = ();
 
     for my $item (qw(useithreads)) {
         next unless $Config{$item} and $Config{$item} eq 'define';
-        push @defines, "-DPERL_\U$item";
+        push @defines, "-D PERL_\U$item";
+    }
+
+    if (my $defines = $self->{config}->{vars}->{defines}) {
+        push @defines, map { "-D $_" } split " ", $defines;
     }
 
     "@defines";
@@ -100,11 +156,11 @@ sub args {
     "-d $vars->{serverroot} -f $vars->{t_conf_file} $dversion $defines";
 }
 
-my %one_process = (1 => '-X', 2 => '-DONE_PROCESS');
+my %one_process = (1 => '-X', 2 => '-D ONE_PROCESS');
 
 sub start_cmd {
     my $self = shift;
-    #XXX: threaded mpm does not respond to SIGTERM with -DONE_PROCESS
+    #XXX: threaded mpm does not respond to SIGTERM with -D ONE_PROCESS
     my $args = $self->args;
     return "$self->{config}->{vars}->{httpd} $args";
 }
@@ -540,7 +596,14 @@ sub start {
             last;
         }
         elsif ($delta > $timeout) {
-            print $preamble, "giving up after $delta secs\n";
+            my $suggestion = $timeout + 300;
+            print $preamble, "not ok\n";
+            error <<EOI;
+giving up after $delta secs. If you think that your system
+is slow or overloaded try again with a longer timeout value.
+by setting the environment variable APACHE_TEST_STARTUP_TIMEOUT
+to a high value (e.g. $suggestion) and repeat the last command.
+EOI
             last;
         }
     }
