@@ -32,6 +32,6 @@
  * @return status
  */
 MP_INLINE int modperl_cgi_header_parse(request_rec *r, char *buffer,
-                                       int *len, const char **body);
+                                       apr_size_t *len, const char **body);
 
 #endif /* MODPERL_CGI_H */
