@@ -228,6 +228,7 @@ typedef struct {
     apr_status_t rc;
     modperl_filter_mode_e mode;
     apr_pool_t *pool;
+    apr_pool_t *temp_pool;
 } modperl_filter_t;
 
 typedef struct {
@@ -260,13 +261,6 @@ typedef struct {
     void *data;
 } modperl_cleanup_data_t;
 
-/* subclass apr_uri_t */
-typedef struct {
-    apr_uri_t uri;
-    apr_pool_t *pool;
-    char *path_info;
-} modperl_uri_t;
-
 typedef struct {
     module *modp;
     const char *cmd_data;
