@@ -106,6 +106,9 @@ char *modperl_file2package(apr_pool_t *p, const char *file);
  */
 char *modperl_coderef2text(pTHX_ apr_pool_t *p, CV *cv);
 
+SV *modperl_apr_array_header2avrv(pTHX_ apr_array_header_t *array);
+apr_array_header_t *modperl_avrv2apr_array_header(pTHX_ apr_pool_t *p,
+                                                  SV *avrv);
 #if defined(MP_TRACE) && defined(APR_HAS_THREADS)
 #define MP_TRACEf_TID   "/tid 0x%lx"
 #define MP_TRACEv_TID   (unsigned long)apr_os_thread_current()
