@@ -16,30 +16,19 @@ use APR::Const -compile => ':common';
 sub handler : FilterRequestHandler {
     my($filter, $bb, $mode, $block, $readbytes) = @_;
 
-    #warn "Called!";
-    my $ba = $filter->r->connection->bucket_alloc;
+    $filter->next->get_brigade($bb, $mode, $block, $readbytes);
 
-    my $ctx_bb = APR::Brigade->new($filter->r->pool, $ba);
+    for (my $b = $bb->first; $b; $b = $bb->next($b)) {
 
-    $filter->next->get_brigade($ctx_bb, $mode, $block, $readbytes);
-
-    while (!$ctx_bb->is_empty) {
-        my $b = $ctx_bb->first;
-
-        $b->remove;
-
-        if ($b->is_eos) {
-            #warn "EOS!!!!";
-            $bb->insert_tail($b);
-            last;
-        }
+        last if $b->is_eos;
 
         if ($b->read(my $data)) {
             #warn"[$data]\n";
-            $b = APR::Bucket->new(scalar reverse $data);
+            my $nb = APR::Bucket->new(scalar reverse $data);
+            $b->insert_before($nb);
+            $b->delete;
+            $b = $nb;
         }
-
-        $bb->insert_tail($b);
     }
 
     Apache::OK;
