@@ -80,7 +80,7 @@ sub merge {
 
 use Apache::RequestRec ();
 use Apache::RequestIO ();
-use Apache::Server ();
+use Apache::ServerRec ();
 use Apache::ServerUtil ();
 use Apache::Module ();
 
