@@ -5,7 +5,7 @@ use warnings FATAL => 'all';
 
 use Apache::Test;
 
-use Apache::Server ();
+use Apache::ServerRec ();
 use Apache::RequestRec ();
 use Apache::RequestIO ();
 
@@ -44,39 +44,20 @@ sub response {
 __END__
 <NoAutoConfig>
     <IfModule mod_proxy.c>
-        # 2.0
-        <IfModule mod_access.c>
-            <Proxy http://@servername@:@port@/*>
+        <Proxy http://@servername@:@port@/*>
+            <IfModule @ACCESS_MODULE@>
                 Order Deny,Allow
                 Deny from all
                 Allow from @servername@
-            </Proxy>
-            ProxyRequests On
+            </IfModule>
+        </Proxy>
 
-            PerlModule TestModules::proxy
-            PerlTransHandler TestModules::proxy::proxy
-            <Location /TestModules__proxy_real>
-                SetHandler modperl
-                PerlResponseHandler TestModules::proxy::response
-            </Location>
-        </IfModule>
-
-        # 2.1
-        <IfModule mod_authz_host.c>
-            <Proxy http://@servername@:@port@/*>
-                Order Deny,Allow
-                Deny from all
-                Allow from @servername@
-            </Proxy>
-            ProxyRequests On
-
-            PerlModule TestModules::proxy
-            PerlTransHandler TestModules::proxy::proxy
-            <Location /TestModules__proxy_real>
-                SetHandler modperl
-                PerlResponseHandler TestModules::proxy::response
-            </Location>
-        </IfModule>
+        PerlModule TestModules::proxy
+        PerlTransHandler TestModules::proxy::proxy
+        <Location /TestModules__proxy_real>
+            SetHandler modperl
+            PerlResponseHandler TestModules::proxy::response
+        </Location>
     </IfModule>
 </NoAutoConfig>
 
