@@ -1,3 +1,18 @@
+/* Copyright 2001-2004 The Apache Software Foundation
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+
 static MP_INLINE
 char *mpxs_apr_uri_unparse(pTHX_
                            apr_uri_t *uptr,
@@ -39,12 +54,19 @@ char *mpxs_APR__URI_port(pTHX_ apr_uri_t *uri, SV *portsv)
     char *port_str = uri->port_str;
 
     if (portsv) {
-        STRLEN len;
-        char *port = SvPV(portsv, len);
-        uri->port_str = apr_pstrndup(((modperl_uri_t *)uri)->pool,
-                                     port, len);
-        uri->port = (int)SvIV(portsv);
+        if (SvOK(portsv)) {
+            STRLEN len;
+            char *port = SvPV(portsv, len);
+            uri->port_str = apr_pstrndup(((modperl_uri_t *)uri)->pool,
+                                         port, len);
+            uri->port = (int)SvIV(portsv);
+        }
+        else {
+            uri->port_str = NULL;
+            uri->port = 0;
+        }
     }
 
     return port_str;
 }
+
