@@ -23,7 +23,7 @@
     const char *bodytext; \
     MP_CGI_HEADER_PARSER_OFF(rcfg); \
     SvPV_force(sv, len);            \
-    modperl_cgi_header_parse(r, SvPVX(sv), &len, &bodytext);        \
+    modperl_cgi_header_parse(r, SvPVX(sv), (apr_size_t*)&len, &bodytext); \
     if (len) {\
         MP_CHECK_WBUCKET_INIT("$r->send_cgi_header"); \
         modperl_wbucket_write(aTHX_ rcfg->wbucket, bodytext, &len); \
